#include <windows.h>
#include <dsound.h>
#include <mmsystem.h>

#define  WindowTitle      "DirectSound-Tutorial"
#define  WindowClassName  "DSWinClass"

LPDIRECTSOUND		lpDirectSound;
LPDIRECTSOUNDBUFFER dsbSound;

bool				running;


bool DSoundInit (HWND hwnd)
{
	if(DirectSoundCreate (NULL, &lpDirectSound, NULL) != DS_OK)
	{
		return (false);
	}

	if(lpDirectSound->SetCooperativeLevel (hwnd, DSSCL_NORMAL) != DS_OK)
	{
		return (false);
	}

	return (true);
}

void ReleaseDSound ()
{
	if (dsbSound != NULL)
	{
		dsbSound->Release();
		dsbSound = NULL;
	}

	if (lpDirectSound != NULL)
	{
		lpDirectSound->Release();
		lpDirectSound = NULL;
	}
}

bool LoadWave (char *filename)
{
	HMMIO wavefile;
	wavefile = mmioOpen (filename, 0, MMIO_READ | MMIO_ALLOCBUF);
	if (wavefile == NULL) 
	{
		return (false);
	}
	
	MMCKINFO parent;
	memset (&parent, 0, sizeof(MMCKINFO));
	parent.fccType = mmioFOURCC ('W', 'A', 'V', 'E');
	mmioDescend (wavefile, &parent, 0, MMIO_FINDRIFF);

	MMCKINFO child;
	memset (&child, 0, sizeof(MMCKINFO));
	child.fccType = mmioFOURCC ('f', 'm', 't', ' ');
	mmioDescend (wavefile, &child, &parent, 0);

	WAVEFORMATEX wavefmt;
	mmioRead (wavefile, (char*)&wavefmt, sizeof(wavefmt));
	if(wavefmt.wFormatTag != WAVE_FORMAT_PCM)
	{
		return (false);
	}

	mmioAscend (wavefile, &child, 0);
	child.ckid = mmioFOURCC ('d', 'a', 't', 'a');
	mmioDescend (wavefile, &child, &parent, MMIO_FINDCHUNK);

	DSBUFFERDESC bufdesc;
	memset (&bufdesc, 0, sizeof(DSBUFFERDESC));
	bufdesc.dwSize = sizeof(DSBUFFERDESC);

	bufdesc.dwFlags = 0;
	bufdesc.dwBufferBytes = child.cksize;
	bufdesc.lpwfxFormat = &wavefmt;
	if((lpDirectSound->CreateSoundBuffer (&bufdesc, &dsbSound, NULL)) != DS_OK)
	{
		return (false);
	}

	void *write1 = 0, *write2 = 0;
	unsigned long length1,length2;
	dsbSound->Lock (0, child.cksize, &write1, &length1, &write2, &length2, 0);
	if(write1 > 0)
		mmioRead (wavefile, (char*)write1, length1);
	if (write2 > 0)
		mmioRead (wavefile, (char*)write2, length2);
	dsbSound->Unlock (write1, length1, write2, length2);

	mmioClose (wavefile, 0);

	return (true);
}

void PlayWave ()
{
	dsbSound->SetCurrentPosition (0);
	dsbSound->Play (0, 0, 0);
}

void StopWave ()
{
	dsbSound->Stop ();
}


LONG FAR WINAPI WndProc (HWND , UINT , UINT , LONG) ;

int WINAPI WinMain (HANDLE hInstance    , HANDLE hPrevInstance ,  
                    LPSTR  lpszCmdParam , int    nCmdShow)  
{
	MSG      msg      ;  
	HWND     hwnd     ;   
	WNDCLASS wndclass ; 
	if (!hPrevInstance)
    {
		wndclass.style         = CS_HREDRAW | CS_VREDRAW  ;
		wndclass.lpfnWndProc   = WndProc ;
		wndclass.cbClsExtra    = 0 ;
		wndclass.cbWndExtra    = 0 ;
		wndclass.hInstance     = hInstance   ;
		wndclass.hIcon         = LoadIcon (NULL , IDI_APPLICATION) ;
		wndclass.hCursor       = LoadCursor (NULL , IDC_ARROW)  ; 
		wndclass.hbrBackground = GetStockObject (LTGRAY_BRUSH)  ;
		wndclass.lpszMenuName  = NULL              ;
		wndclass.lpszClassName = "Test" ; 
		RegisterClass (&wndclass) ;    
	}
	hwnd = CreateWindow ("Test" , 
                         "Test" , 
                         WS_OVERLAPPEDWINDOW , 
			             CW_USEDEFAULT , 
						 CW_USEDEFAULT ,
                         CW_USEDEFAULT , 
                         CW_USEDEFAULT ,     
                         NULL , NULL , 
                         hInstance , NULL) ;     
	ShowWindow   (hwnd , nCmdShow) ;  
	UpdateWindow (hwnd) ;

	DSoundInit (hwnd);

	LoadWave ("ctmelody.wav");

	running=true;

	while(running)
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
			if (msg.message == WM_QUIT) break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	} 

	ReleaseDSound ();

	return msg.wParam ;
}


LONG FAR WINAPI WndProc (HWND hwnd , UINT message , 
                         UINT wParam , LONG lParam) 
{ 
	switch (message)
	{
		case WM_KEYDOWN:
		{
			switch (wParam)
			{
			case VK_ESCAPE:
				{
				running=false;
				return 0;
				}
			case VK_F1:
				{
					PlayWave ();
					return 0;
				}
			case VK_F2:
				{
					StopWave ();
					return 0;
				}

			} break;
		}
		case WM_DESTROY:
		{
			PostQuitMessage (0) ; 
			return 0 ;
		} break;
	} // switch (message)
	return DefWindowProc (hwnd , message , wParam , lParam) ;
} // WndProc 
